package com.sparsity.lgcre;

import androidx.annotation.NonNull;
import android.content.Context;
import android.os.Bundle;
import android.provider.Settings;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;


/** LgcrePlugin */
public class LgcrePlugin implements FlutterPlugin, MethodCallHandler {
	/// The MethodChannel that will the communication between Flutter and native Android
	///
	/// This local reference serves to register the plugin with the Flutter Engine and unregister it
	/// when the Flutter Engine is detached from the Activity
	private MethodChannel channel;
	private Context       ctx;

	@Override
	public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
		channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "lgcre");
		channel.setMethodCallHandler(this);
		ctx = flutterPluginBinding.getApplicationContext();
	}

	@Override
	public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
		if (call.method.equals("getPlatformVersion")) {
			result.success("Android " + android.os.Build.VERSION.RELEASE);
		}
		else if(call.method.equals("init"))
		{
			String gdbDir = ctx.getDir("lgcre_data", Context.MODE_PRIVATE).getAbsolutePath();

			Map<String,String> options = call.arguments();
			options.put("lgcre.workspace",gdbDir);

			try
			{
				LGCRE.init(ctx,options);
				result.success(null);
			}
			catch(java.lang.RuntimeException e)
			{
				result.error("INTERNAL ERROR", e.getMessage(), null);
				return;
			}
		}
		else if(call.method.equals("reset"))
		{
      LGCRETx tx = null;
			try{
				tx = LGCRE.beginTx();
				LGCRE.reset(tx);
        LGCRE.commitTx(tx);
				result.success(null);
			}
			catch(java.lang.RuntimeException e)
			{
        if(tx != null)
          LGCRE.rollbackTx(tx);
				result.error("INTERNAL ERROR", e.getMessage(), null);
				return;
			}
		}
		else if(call.method.equals("release"))
		{
			try
			{
				LGCRE.release();
				result.success(null);
			}
			catch(java.lang.RuntimeException e)
			{
				result.error("INTERNAL ERROR", e.getMessage(), null);
				return;
			}
		}
		else if(call.method.equals("run"))
		{
			LGCREQuery query = new LGCREQuery();
			Map<String,Object> arguments = call.arguments();
			for(String key : arguments.keySet())
			{
				if(key.compareTo("pois") != 0)
				{
					result.error("UNKNOWN_DATA_KEY", "Data key "+key+" is unknown", null);
					return;
				}
			}

			List<Map<String,Object>> pois = (List<Map<String,Object>>)arguments.get("pois");
			for(Map<String,Object> entry : pois)
			{
				for(String key : entry.keySet())
				{
					if(key.compareTo("poi") != 0 &&
							key.compareTo("categories") != 0)
					{
						result.error("UNKNOWN_DATA_KEY", "Data key "+key+" is unknown", null);
						return;
					}
				}

				String poi = (String)entry.get("poi");
				List<String> categories = (List<String>)entry.get("categories");
				for(String category : categories)
				{
					query.setPoiCategory(poi, category);
				}
			}

      LGCRETx tx = null;
			try
			{
				tx = LGCRE.beginTx();
				List ret = new ArrayList();
				LGCREQueryResultEntry[] entries = LGCRE.run(tx, query);
        LGCRE.commitTx(tx);
				if(entries != null)
				{
					int count= 0;
					for(LGCREQueryResultEntry entry : entries)
					{
						StringBuilder strBuilder = new StringBuilder();
						strBuilder.append("{\"poi\" : \""+entry.getPoi()+"\", ");
						strBuilder.append("\"score\" : "+entry.getScore()+"}");
						ret.add(strBuilder.toString());
						count++;
					}
				}
				result.success(ret);
				return;

			}
			catch(java.lang.RuntimeException e)
			{
        if(tx != null)
          LGCRE.rollbackTx(tx);
				result.error("INTERNAL ERROR", e.getMessage(), null);
				return;
			}
		}
		else if(call.method.equals("addData"))
		{
      LGCRETx tx = null;
			try
			{
				tx = LGCRE.beginTx();
				Map<String,Object> arguments = call.arguments();

				for(String key : arguments.keySet())
				{
					if(key.compareTo("pois") != 0 &&
							key.compareTo("categories") != 0 &&
							key.compareTo("links") != 0 &&
							key.compareTo("links_categories") != 0 &&
							key.compareTo("links_poi_category") != 0 &&
							key.compareTo("person") != 0)
					{
						result.error("UNKNOWN_DATA_KEY", "Data key "+key+" is unknown", null);
						return;
					}
				}

				List<String> pois = (List<String>)arguments.get("pois");
				if(pois != null)
				{
					for (String poi : pois)
					{
						LGCRE.addPoi(tx, poi);
					}
				}

				List<String> categories = (List<String>)arguments.get("categories");
				if(categories != null)
				{
					for (String category : categories)
					{
						LGCRE.addCategory(tx, category);
					}
				}

				List<Map<String,String>> link_categories = (List<Map<String, String>>)arguments.get("links_categories");
				if(link_categories != null)
				{
					for(Map<String, String> link : link_categories)
					{
						for(String key : link.keySet())
						{
							if(key.compareTo("category1") != 0 &&
									key.compareTo("category2") != 0)
							{
								result.error("UNKNOWN_DATA_KEY", "Data key "+key+" in links_categories entry is unknown", null);
								return;
							}
						}
						LGCRE.addLinkCategoryCategory(tx, link.get("category1"), link.get("category2"));
					}
				}

				List<Map<String,String>> link_poi_categories = (List<Map<String, String>>) arguments.get("links_poi_category");
				if(link_poi_categories != null)
				{

					for(Map<String, String> link : link_poi_categories)
					{
						for(String key : link.keySet())
						{
							if(key.compareTo("poi") != 0 &&
									key.compareTo("category") != 0)
							{
								result.error("UNKNOWN_DATA_KEY", "Data key "+key+" in links_poi_category entry is unknown", null);
								return;
							}
						}
						LGCRE.addLinkPoiCategory(tx, link.get("poi"), link.get("category"));
					}
				}

				Map<String,Object> person_data = (Map<String, Object>)arguments.get("person");
				if(person_data != null)
				{

					for(String key : person_data.keySet())
					{
						if(key.compareTo("pois") != 0 &&
								key.compareTo("categories") != 0)
						{
							result.error("UNKNOWN_DATA_KEY", "Data key "+key+" in person entry is unknown", null);
							return;
						}
					}

					List<String> person_pois = (List<String>)person_data.get("pois");
					if(person_pois != null)
					{
						for (String poi : person_pois)
						{
							LGCRE.addLinkPersonPoi(tx, poi);
						}
					}

					List<String> person_categories = (List<String>)person_data.get("categories");
					if(person_categories != null)
					{
						for (String category : person_categories)
						{
							LGCRE.addLinkPersonCategory(tx, category);
						}
					}
				}
        LGCRE.commitTx(tx);
				result.success(null);
			}
			catch(java.lang.RuntimeException e)
			{
        if(tx != null)
          LGCRE.rollbackTx(tx);
				result.error("INTERNAL ERROR", e.getMessage(), null);
				return;
			}

		}
		else if(call.method.equals("removeData"))
		{
      LGCRETx tx = null;
			try
			{
				tx = LGCRE.beginTx();
				Map<String,Object> arguments = call.arguments();

				for(String key : arguments.keySet())
				{
					if(key.compareTo("pois") != 0 &&
							key.compareTo("categories") != 0 &&
							key.compareTo("links") != 0 &&
							key.compareTo("links_categories") != 0 &&
							key.compareTo("links_poi_category") != 0 &&
							key.compareTo("person") != 0)
					{
						result.error("UNKNOWN_DATA_KEY", "Data key "+key+" is unknown", null);
						return;
					}
				}

				List<Map<String,String>> link_categories = (List<Map<String, String>>)arguments.get("links_categories");
				if(link_categories != null)
				{
					for(Map<String, String> link : link_categories)
					{
						for(String key : link.keySet())
						{
							if(key.compareTo("category1") != 0 &&
									key.compareTo("category2") != 0)
							{
								result.error("UNKNOWN_DATA_KEY", "Data key "+key+" in links_categories entry is unknown", null);
								return;
							}
						}
						LGCRE.removeLinkCategoryCategory(tx, link.get("category1"), link.get("category2"));
					}
				}

				List<Map<String,String>> link_poi_categories = (List<Map<String, String>>) arguments.get("links_poi_category");
				if(link_poi_categories != null)
				{

					for(Map<String, String> link : link_poi_categories)
					{
						for(String key : link.keySet())
						{
							if(key.compareTo("poi") != 0 &&
									key.compareTo("category") != 0)
							{
								result.error("UNKNOWN_DATA_KEY", "Data key "+key+" in links_poi_category entry is unknown", null);
								return;
							}
						}
						LGCRE.removeLinkPoiCategory(tx, link.get("poi"), link.get("category"));
					}
				}

				Map<String,Object> person_data = (Map<String, Object>)arguments.get("person");
				if(person_data != null)
				{

					for(String key : person_data.keySet())
					{
						if(key.compareTo("pois") != 0 &&
								key.compareTo("categories") != 0)
						{
							result.error("UNKNOWN_DATA_KEY", "Data key "+key+" in person entry is unknown", null);
							return;
						}
					}

					List<String> person_pois = (List<String>)person_data.get("pois");
					if(person_pois != null)
					{
						for (String poi : person_pois)
						{
							LGCRE.removeLinkPersonPoi(tx, poi);
						}
					}

					List<String> person_categories = (List<String>)person_data.get("categories");
					if(person_categories != null)
					{
						for (String category : person_categories)
						{
							LGCRE.removeLinkPersonCategory(tx, category);
						}
					}
				}


				List<String> pois = (List<String>)arguments.get("pois");
				if(pois != null)
				{
					for (String poi : pois)
					{
						LGCRE.removePoi(tx, poi);
					}
				}

				List<String> categories = (List<String>)arguments.get("categories");
				if(categories != null)
				{
					for (String category : categories)
					{
						LGCRE.removeCategory(tx, category);
					}
				}
        LGCRE.commitTx(tx);
				result.success(null);
			}
			catch(java.lang.RuntimeException e)
			{
        if(tx != null)
          LGCRE.rollbackTx(tx);
				result.error("INTERNAL ERROR", e.getMessage(), null);
				return;
			}

		}
		else
		{
			result.notImplemented();
			return;
		}
	}

	@Override
	public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
		channel.setMethodCallHandler(null);
	}
}
