#import "LgcrePlugin.h"
#import <LGCRE/LGCRE.h>

@implementation LgcrePlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
  methodChannelWithName:@"lgcre"
        binaryMessenger:[registrar messenger]];
  LgcrePlugin* instance = [[LgcrePlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
     result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } 
  else if ([@"init" isEqualToString:call.method]) {
    @try 
    {

      NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);

      NSString *libraryPath = ([paths count] > 0) ? [paths objectAtIndex:0] : nil;
      NSString *workspace = [libraryPath stringByAppendingPathComponent:@"Lgcre"];
      NSFileManager *fileManager = [NSFileManager defaultManager];
      [fileManager createDirectoryAtPath: workspace
             withIntermediateDirectories:TRUE
                              attributes:nil
                                   error:nil];
      NSMutableDictionary* options = [call arguments];
      [options setObject: workspace forKey: @"lgcre.workspace"];
      LGCREInitWithOptions(options);
      result(nil);
    }
    @catch(NSException* exception)
    {
      result([FlutterError errorWithCode: @"Internal Error" 
                                 message:[exception reason]
                                 details:nil]);
    }
  } 
  else if ([@"reset" isEqualToString:call.method]) {
    LGCRETx* tx = nil;
    @try
    {
      tx = LGCREBeginTx();
      LGCREReset(tx);
      LGCRECommitTx(tx);
      result(nil);

    }
    @catch(NSException* exception)
    {
      if (tx != nil)
        LGCRERollbackTx(tx);
      result([FlutterError errorWithCode: @"Internal Error" 
                                 message:[exception reason]
                                 details:nil]);
    }
  } 
  else if ([@"release" isEqualToString:call.method]) {
    LGCRERelease();
    result(nil);
  } 
  else if ([@"run" isEqualToString:call.method]) {
    LGCREQuery* query = [[LGCREQuery alloc] init];
    NSDictionary* arguments = [call arguments];
    for(NSString* key in arguments)
    {
      if(![key isEqualToString:@"pois"])
      {
        result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ is unknown", key]
                                   message:nil
                                   details:nil]);
        return;

      }
    }

    NSArray* pois = (NSArray*)arguments[@"pois"];
    for(NSDictionary* entry in pois)
    {
      for(NSString* key in entry)
      {
        if(![key isEqualToString:@"poi"] &&
           ![key isEqualToString:@"categories"])
        {
          result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ is unknown", key]
                                     message:nil
                                     details:nil]);
          return;
        }
      }

      NSString* poi = (NSString*)entry[@"poi"];
      NSArray* categories = (NSArray*)entry[@"categories"];
      for(NSString* category in categories)
      {
        [query setPoiCategory: poi cat: category];
      }
    }

    LGCRETx* tx = nil;
    NSMutableArray* ret = [[NSMutableArray alloc] init];
    @try
    {
      tx = LGCREBeginTx();
      NSArray* entries = LGCRERun(tx,query);
      LGCRECommitTx(tx);
      query= nil;
      if(entries != nil)
      {
        int count= 0;
        for(LGCREQueryResultEntry* entry in entries)
        {
          NSString* str = [NSString stringWithFormat:@"{\"poi\" : \"%@\", \"score\" : %f} ",
          [entry getPoi],
          [entry getCategory]
          ];

          [ret addObject: str];
          count++;
        }
      }
      result(ret);
    }
    @catch(NSException* exception)
    {
      if (tx != nil)
        LGCRERollbackTx(tx);
      result([FlutterError errorWithCode: @"Internal Error"
                                 message:[exception reason]
                                 details:nil]);
    }
  } 
  else if ([@"addData" isEqualToString:call.method])
  {

    LGCRETx* tx = nil;
    @try
    {
      tx = LGCREBeginTx();
      NSDictionary* arguments = [call arguments];

      for (NSString *key in arguments)
      {
        if(![key isEqualToString:@"pois"] &&
           ![key isEqualToString:@"categories"] &&
           ![key isEqualToString:@"links"] &&
           ![key isEqualToString:@"links_categories"] &&
           ![key isEqualToString:@"links_poi_category"] &&
           ![key isEqualToString:@"person"])
        {
          result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ is unknown", key]
                                     message:nil
                                     details:nil]);
          return;
        }
      }

      NSArray* pois = (NSArray*)arguments[@"pois"];
      if(pois != nil)
      {
        for (NSString* poi in pois)
        {
          LGCREAddPoi(tx, poi);
        }
      }

      NSArray* categories = (NSArray*)arguments[@"categories"];
      if(categories != nil)
      {
        for (NSString* category in categories)
        {
          LGCREAddCategory(tx, category);
        }
      }

      NSDictionary* link_categories = (NSDictionary*)arguments[@"links_categories"];
      if(link_categories != nil)
      {
        for(NSDictionary* link in link_categories)
        {
          for(NSString* key in link)
          {
            if(![key isEqualToString:@"category1"] &&
               ![key isEqualToString:@"category2"])
            {
              result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in links_categories is unknown", key]
                                         message:nil
                                         details:nil]);
              return;
            }
          }
          LGCREAddLinkCategoryCategory(tx, link[@"category1"], link[@"category2"]);
        }
      }

      NSDictionary* link_poi_categories = (NSDictionary*) arguments[@"links_poi_category"];
      if(link_poi_categories != nil)
      {
        for(NSDictionary* link in link_poi_categories)
        {
          for(NSString* key in link)
          {
            if(![key isEqualToString:@"poi"] &&
               ![key isEqualToString:@"category"])
            {
              result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in links_poi_categories is unknown", key]
                                         message:nil
                                         details:nil]);
              return;
            }
          }
          LGCREAddLinkPoiCategory(tx, link[@"poi"], link[@"category"]);
        }
      }

      NSDictionary* person_data = (NSDictionary*)arguments[@"person"];
      if(person_data !=  nil)
      {
        for(NSString* key in person_data)
        {
          if(![key isEqualToString:@"pois"] &&
             ![key isEqualToString:@"categories"])
          {
            result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in person entry is unknown", key]
                                       message:nil
                                       details:nil]);
            return;
          }
        }

        NSArray* person_pois = (NSArray*)person_data[@"pois"];
        if(person_pois != nil)
        {
          for (NSString* poi in person_pois)
          {
            LGCREAddLinkPersonPoi(tx, poi);
          }
        }

        NSArray* person_categories = (NSArray*)person_data[@"categories"];
        if(person_categories != nil)
        {
          for (NSString* category in person_categories)
          {
            LGCREAddLinkPersonCategory(tx, category);
          }
        }
      }
      LGCRECommitTx(tx);
      result(nil);
    }
    @catch(NSException* exception)
    {
      if (tx != nil)
        LGCRERollbackTx(tx);
      result([FlutterError errorWithCode: @"Internal Error" 
                                 message:[exception reason]
                                 details:nil]);
    }
  } 
  else if ([@"removeData" isEqualToString:call.method]) {

    LGCRETx* tx = nil;
    @try
    {
      tx = LGCREBeginTx();
      NSDictionary* arguments = [call arguments];

      for (NSString *key in arguments)
      {
        if(![key isEqualToString:@"pois"] &&
           ![key isEqualToString:@"categories"] &&
           ![key isEqualToString:@"links"] &&
           ![key isEqualToString:@"links_categories"] &&
           ![key isEqualToString:@"links_poi_category"] &&
           ![key isEqualToString:@"person"])
        {
          result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ is unknown", key]
                                     message:nil
                                     details:nil]);
          return;
        }
      }


      NSDictionary* link_categories = (NSDictionary*)arguments[@"links_categories"];
      if(link_categories != nil)
      {
        for(NSDictionary* link in link_categories)
        {
          for(NSString* key in link)
          {
            if(![key isEqualToString:@"category1"] &&
               ![key isEqualToString:@"category2"])
            {
              result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in links_categories is unknown", key]
                                         message:nil
                                         details:nil]);
              return;
            }
          }
          LGCRERemoveLinkCategoryCategory(tx, link[@"category1"], link[@"category2"]);
        }
      }

      NSDictionary* link_poi_categories = (NSDictionary*) arguments[@"links_poi_category"];
      if(link_poi_categories != nil)
      {
        for(NSDictionary* link in link_poi_categories)
        {
          for(NSString* key in link)
          {
            if(![key isEqualToString:@"poi"] &&
               ![key isEqualToString:@"category"])
            {
              result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in links_poi_categories is unknown", key]
                                         message:nil
                                         details:nil]);
              return;
            }
          }
          LGCRERemoveLinkPoiCategory(tx, link[@"poi"], link[@"category"]);
        }
      }

      NSDictionary* person_data = (NSDictionary*)arguments[@"person"];
      if(person_data !=  nil)
      {
        for(NSString* key in person_data)
        {
          if(![key isEqualToString:@"pois"] &&
             ![key isEqualToString:@"categories"])
          {
            result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in person entry is unknown", key]
                                       message:nil
                                       details:nil]);
            return;
          }
        }

        NSArray* person_pois = (NSArray*)person_data[@"pois"];
        if(person_pois != nil)
        {
          for (NSString* poi in person_pois)
          {
            LGCRERemoveLinkPersonPoi(tx, poi);
          }
        }

        NSArray* person_categories = (NSArray*)person_data[@"categories"];
        if(person_categories != nil)
        {
          for (NSString* category in person_categories)
          {
            LGCRERemoveLinkPersonCategory(tx, category);
          }
        }
      }

      NSArray* pois = (NSArray*)arguments[@"pois"];
      if(pois != nil)
      {
        for (NSString* poi in pois)
        {
          LGCRERemovePoi(tx, poi);
        }
      }

      NSArray* categories = (NSArray*)arguments[@"categories"];
      if(categories != nil)
      {
        for (NSString* category in categories)
        {
          LGCRERemoveCategory(tx, category);
        }
      }

      LGCRECommitTx(tx);
      result(nil);
    }
    @catch(NSException* exception)
    {
      if (tx != nil)
        LGCRERollbackTx(tx);
      result([FlutterError errorWithCode: @"Internal Error" 
                                 message:[exception reason]
                                 details:nil]);
    }
  } 
  else {
    result(FlutterMethodNotImplemented);
  }
}

@end
